@@ -30,8 +30,8 @@
 RData * check_target(const OPArg *, const VList *drive_list, const SDir *);
 static inline VList * copy_arg_link(const OPArg *, const SDir *, CPInfo *, CPDD *, VList *file_list);
 static inline VList * copy_arg_file(const VList *drive_list, const char src[], const char dst[], const OPArg *, const SDir *, CPInfo *, CPDD *, VList *file_list);
-static inline VList * copy_arg_dir(const char *tmp_src, const char *tmp_dst, const _Bool src_current_flag, const VList *drive_list, const OPArg *, const SDir *, CPInfo *, CPDD *, VList *file_list);
-static void check_overwrite(const OPArg *, const SDir *, CPInfo *, CPDD *);
+static inline VList * copy_arg_dir(const char *tmp_src, const char *tmp_dst, const _Bool is_src_current, const VList *drive_list, const OPArg *, const SDir *, CPInfo *, CPDD *, VList *file_list);
+static void check_overwrite(const OPArg *, const SDir *, CPInfo *, CPDD *, const char type);
 static void check_write_file_or_create_link(const OPArg *, const SDir *, CPInfo *, CPDD *);
 static inline _Bool check_parent(const char *src, const char *dst);
 static inline struct stat * make_directory(const OPArg *, CPInfo *, CPDD *);
@@ -94,7 +94,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 
 		if((acount == SINGULAR) && (*s == '.'))
 		{
-			_Bool current_flag = true;
+			_Bool is_current = true;
 			s++;
 			char t = G_DIR_SEPARATOR;
 
@@ -120,12 +120,12 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 				}
 				else
 				{
-					current_flag = false;
+					is_current = false;
 					break;
 				}
 			}
 
-			if(current_flag == true)
+			if(is_current == true)
 			{
 				errno = 0;
 
@@ -197,7 +197,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 
 	for(; loop < loop_dst; loop++)
 	{
-		_Bool src_current_flag = false;
+		_Bool is_src_current = false;
 
 		/* strcmpは等しければ0を返す */
 		if(strcmp(oparg->argv[loop], "..") == 0)
@@ -207,7 +207,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 		}
 		else if(strcmp(oparg->argv[loop], ".") == 0)
 		{
-			src_current_flag = true;
+			is_src_current = true;
 		}
 		/*
 		 * メモ
@@ -215,11 +215,11 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 		 * 引数が..だとエラー。
 		 */
 
-		_Bool link_flag = false;
+		_Bool is_link = false;
 
 		if(g_file_test(oparg->argv[loop], G_FILE_TEST_IS_SYMLINK) == TRUE)
 		{
-			link_flag = true;
+			is_link = true;
 			strcpy(tmp_src, oparg->argv[loop]);
 		}
 		else
@@ -258,7 +258,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 				switch(acount)
 				{
 				case SINGULAR:
-					if(link_flag == true)
+					if(is_link == true)
 					{
 						fprintf(stderr, "リンク (%s) に対するコピー先が不正 (ファイル) です\n", tmp_src);
 						exit(EXIT_FAILURE);
@@ -289,7 +289,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 			{
 				cpinfo->dst = get_dst_plus_srcname(tmp_src, tmp_dst);
 
-				if(link_flag == true)
+				if(is_link == true)
 				{
 					file_list = copy_arg_link(oparg, sdir, cpinfo, cpdd, file_list);
 				}
@@ -299,7 +299,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 				}
 				else if(g_file_test(tmp_src, G_FILE_TEST_IS_DIR) == TRUE)
 				{
-					file_list = copy_arg_dir(tmp_src, tmp_dst, src_current_flag, drive_list, oparg, sdir, cpinfo, cpdd, file_list);
+					file_list = copy_arg_dir(tmp_src, tmp_dst, is_src_current, drive_list, oparg, sdir, cpinfo, cpdd, file_list);
 				}
 				else
 				{
@@ -340,7 +340,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 				cpinfo->dst = get_dst_plus_srcname(oparg->argv[loop_dst], tmp_dst);
 			}
 
-			if(link_flag == true)
+			if(is_link == true)
 			{
 				file_list = copy_arg_link(oparg, sdir, cpinfo, cpdd, file_list);
 			}
@@ -350,7 +350,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 			}
 			else if(g_file_test(tmp_src, G_FILE_TEST_IS_DIR) == TRUE)
 			{
-				file_list = copy_arg_dir(tmp_src, tmp_dst, src_current_flag, drive_list, oparg, sdir, cpinfo, cpdd, file_list);
+				file_list = copy_arg_dir(tmp_src, tmp_dst, is_src_current, drive_list, oparg, sdir, cpinfo, cpdd, file_list);
 			}
 			else
 			{
@@ -375,7 +375,7 @@ RData * check_target(const OPArg *oparg, const VList *drive_list, const SDir *sd
 static inline VList * copy_arg_link(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd, VList *file_list)
 {
 	count_plus_link(cpinfo, cpdd);
-	check_overwrite(oparg, sdir, cpinfo, cpdd);
+	check_overwrite(oparg, sdir, cpinfo, cpdd, 'f');
 	file_list = append_list((void *)cpinfo, file_list);
 	return file_list;
 }
@@ -386,20 +386,20 @@ static inline VList * copy_arg_file(const VList *drive_list, const char tmp_src[
 {
 	count_plus_file(cpinfo, cpdd);
 	set_same_or_different(drive_list, tmp_src, tmp_dst, oparg, cpinfo);
-	check_overwrite(oparg, sdir, cpinfo, cpdd);
+	check_overwrite(oparg, sdir, cpinfo, cpdd, 'f');
 	file_list = append_list((void *)cpinfo, file_list);
 	return file_list;
 }
 
 /*******************************************************************************
 *******************************************************************************/
-static inline VList * copy_arg_dir(const char *tmp_src, const char *tmp_dst, const _Bool src_current_flag, const VList *drive_list, const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd, VList *file_list)
+static inline VList * copy_arg_dir(const char *tmp_src, const char *tmp_dst, const _Bool is_src_current, const VList *drive_list, const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd, VList *file_list)
 {
-	_Bool parent_flag = check_parent(tmp_src, tmp_dst);
+	_Bool is_parent = check_parent(tmp_src, tmp_dst);
 
-	if(parent_flag == true)
+	if(is_parent == true)
 	{
-		if(src_current_flag == true)
+		if(is_src_current == true)
 		{
 			fprintf(stderr, "`.' をスキップします\n");
 		}
@@ -420,7 +420,7 @@ static inline VList * copy_arg_dir(const char *tmp_src, const char *tmp_dst, con
 
 		struct stat *stat_buf;
 
-		if(src_current_flag == false)
+		if(is_src_current == false)
 		{
 			count_plus_dir(cpinfo, cpdd);
 			stat_buf = make_directory(oparg, cpinfo, cpdd);
@@ -436,7 +436,7 @@ static inline VList * copy_arg_dir(const char *tmp_src, const char *tmp_dst, con
 
 		file_list = dir_traverse(oparg, sdir, file_list, cpinfo, cpdd);
 
-		if((src_current_flag == false) && (cpinfo->dst_para == WRITE))
+		if((is_src_current == false) && (cpinfo->dst_para == WRITE))
 		{
 			ch_time_and_mod_dir(cpinfo, stat_buf, cpdd);
 		}
@@ -458,11 +458,11 @@ static inline VList * copy_arg_dir(const char *tmp_src, const char *tmp_dst, con
 		this_dir = NULL;\
 	}\
 }
-#define DO_NOT_OVERWRITE_FLAG { cpinfo->dst_para = SKIP; }
+#define DO_NOT_OVERWRITE { cpinfo->dst_para = SKIP; }
 
 /*******************************************************************************
 *******************************************************************************/
-static void check_overwrite(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd)
+static void check_overwrite(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd, const char type)
 {
 	enum I_OverWrite_Type
 	{
@@ -471,14 +471,14 @@ static void check_overwrite(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo
 		THIS_DIR_NO,
 	};
 
-	static _Bool info_flag;
+	static _Bool is_info;
 	static enum I_OverWrite_Type iowt;
 	static OPFlag I;
 	static OPFlag WMODE;
 
-	if(info_flag == false)
+	if(is_info == false)
 	{
-		info_flag = true;
+		is_info = true;
 		iowt = INFO;
 
 		/* 変更される可能性があるオプションはローカル変数に入れておく */
@@ -515,8 +515,28 @@ INPUT:
 			{
 			case INFO:
 				printf("overwrite ? `%s'\n", cpinfo->dst);
-				puts("y: Yes, a: Yes (All), d: Yes (Only this folder),");
-				puts("n: No,  z: No (All),  o: No (Only this folder)");
+				printf("y: Yes,  a: Yes (All),");
+
+				if(type == 'd')
+				{
+					puts("  d: Yes (Only this folder),");
+				}
+				else
+				{
+					puts("");
+				}
+
+				printf("n:  No,  z:  No (All),");
+
+				if(type == 'd')
+				{
+					puts("  o:  No (Only this folder),");
+				}
+				else
+				{
+					puts("");
+				}
+
 				printf(" : ");
 
 				char stdin_buf[NAME_MAX];
@@ -540,6 +560,11 @@ INPUT:
 					goto INPUT;
 				}
 
+				if((type != 'd') && ((stdin_buf[0] == 'd') || (stdin_buf[0] == 'o')))
+				{
+					goto INPUT;
+				}
+
 				/* このフォルダで○○の時にフォルダパスを記憶しておく変数なのでstatic必須 */
 				static char *this_dir;
 
@@ -564,18 +589,18 @@ INPUT:
 					break;
 
 				case 'n':	/* 上書きしない */
-					DO_NOT_OVERWRITE_FLAG
+					DO_NOT_OVERWRITE
 					break;
 
 				case 'o':	/* このフォルダでは上書きしない */
-					DO_NOT_OVERWRITE_FLAG
+					DO_NOT_OVERWRITE
 					iowt = THIS_DIR_NO;
 					FREE_THIS_DIR
 					MAKE_THIS_DIR
 					break;
 
 				case 'z':	/* 全て上書きしない */
-					DO_NOT_OVERWRITE_FLAG
+					DO_NOT_OVERWRITE
 					I = NOT;	/* 上書き確認モード無効 */
 					WMODE = NO_OVERWRITE;	/* 非上書きモード */
 					break;
@@ -606,7 +631,7 @@ INPUT:
 
 					if(b == true)
 					{
-						DO_NOT_OVERWRITE_FLAG
+						DO_NOT_OVERWRITE
 					}
 					else
 					{
@@ -623,7 +648,7 @@ INPUT:
 		{
 			if(WMODE == NO_OVERWRITE)
 			{
-				DO_NOT_OVERWRITE_FLAG
+				DO_NOT_OVERWRITE
 			}
 			else
 			{
@@ -710,7 +735,7 @@ static inline struct stat * make_directory(const OPArg *oparg, CPInfo *cpinfo, C
 	if(g_file_test(cpinfo->dst, G_FILE_TEST_EXISTS) == TRUE)
 	{
 		/* フォルダの作成は行わない */
-		DO_NOT_OVERWRITE_FLAG
+		DO_NOT_OVERWRITE
 	}
 	else if(mkdir(cpinfo->dst, 0755) == -1)
 	{
@@ -731,7 +756,11 @@ static inline struct stat * make_directory(const OPArg *oparg, CPInfo *cpinfo, C
 		}
 
 		cpinfo->dst_para = WRITE;
-		cpdd->mk_dir_count++;
+
+		if(cpdd->mk_dir_count <= ULLONG_MAX)
+		{
+			cpdd->mk_dir_count++;
+		}
 	}
 
 	struct stat *stat_buf = xmalloc(sizeof(struct stat));
@@ -754,7 +783,7 @@ static inline struct stat * make_directory(const OPArg *oparg, CPInfo *cpinfo, C
 #define TYPE_IS_LINK \
 {\
 	count_plus_link(cpinfo, cpdd);\
-	check_overwrite(oparg, sdir, cpinfo, cpdd);\
+	check_overwrite(oparg, sdir, cpinfo, cpdd, 'd');\
 	file_list = append_list((void *)cpinfo, file_list);\
 }
 
@@ -762,7 +791,7 @@ static inline struct stat * make_directory(const OPArg *oparg, CPInfo *cpinfo, C
 {\
 	count_plus_file(cpinfo, cpdd);\
 	cpinfo->copy_mode = cpinfo_tmp->copy_mode;\
-	check_overwrite(oparg, sdir, cpinfo, cpdd);\
+	check_overwrite(oparg, sdir, cpinfo, cpdd, 'd');\
 	file_list = append_list((void *)cpinfo, file_list);\
 }
 
@@ -775,7 +804,7 @@ static inline struct stat * make_directory(const OPArg *oparg, CPInfo *cpinfo, C
 	/* 再帰 */ \
 	file_list = dir_traverse(oparg, sdir, file_list, cpinfo, cpdd);\
 	if(cpinfo->dst_para == WRITE) {ch_time_and_mod_dir(cpinfo, stat_buf, cpdd); }\
-	if(oparg->M == MOVE) { REMOVE_SOURCE_RMDIR }\
+	if((oparg->M == MOVE) && ((oparg->CHECK_MODE == BEFORE) || (oparg->CHECK_MODE == NOT))) { REMOVE_SOURCE_RMDIR }\
 }
 
 /*******************************************************************************
@@ -971,7 +1000,11 @@ BASENAME_END:
 static inline void count_plus_link(CPInfo *cpinfo, CPDD *cpdd)
 {
 	cpinfo->src_type = SYMBOLICLINK;
-	cpdd->link_count++;
+
+	if(cpdd->link_count <= ULLONG_MAX)
+	{
+		cpdd->link_count++;
+	}
 }
 
 /******************************************************************************
@@ -979,7 +1012,11 @@ static inline void count_plus_link(CPInfo *cpinfo, CPDD *cpdd)
 static inline void count_plus_file(CPInfo *cpinfo, CPDD *cpdd)
 {
 	cpinfo->src_type = REGULAR;
-	cpdd->file_count++;
+
+	if(cpdd->file_count <= ULLONG_MAX)
+	{
+		cpdd->file_count++;
+	}
 }
 
 /******************************************************************************
@@ -987,7 +1024,11 @@ static inline void count_plus_file(CPInfo *cpinfo, CPDD *cpdd)
 static inline void count_plus_dir(CPInfo *cpinfo, CPDD *cpdd)
 {
 	cpinfo->src_type = DIRECTORY;
-	cpdd->dir_count++;
+
+	if(cpdd->dir_count <= ULLONG_MAX)
+	{
+		cpdd->dir_count++;
+	}
 }
 
 /*******************************************************************************
@@ -999,7 +1040,7 @@ static inline void ch_time_and_mod_dir(const CPInfo *cpinfo, const struct stat *
 	if(chmod(cpinfo->dst, stat_buf->st_mode) == -1)
 	{
 		print_error("chmod", __FILE__, __LINE__, cpdd);
-		fprintf(stderr, "フォルダ情報変更エラーです (%s) \n", cpinfo->dst);
+		fprintf(stderr, "error : %s\n", cpinfo->dst);
 	}
 
 /*
@@ -1012,14 +1053,18 @@ static inline void ch_time_and_mod_dir(const CPInfo *cpinfo, const struct stat *
 
 	chown(cpinfo->dst, stat_buf->st_uid, stat_buf->st_gid);
 
-	struct utimbuf times;
-	times.actime  = stat_buf->st_atime;
-	times.modtime = stat_buf->st_mtime;
+	struct timespec time_s[2];
+	time_s[0].tv_sec = stat_buf->st_atime;
+	time_s[1].tv_sec = stat_buf->st_mtime;
 
-	if(utime(cpinfo->dst, &times) == -1)
+	time_s[0].tv_nsec = stat_buf->st_atim.tv_nsec;
+	time_s[1].tv_nsec = stat_buf->st_mtim.tv_nsec;
+
+	/* 第四引数は0かAT_SYMLINK_NOFOLLOW */
+	if(utimensat(AT_FDCWD, cpinfo->dst, time_s, AT_SYMLINK_NOFOLLOW) == -1)
 	{
-		print_error("utime", __FILE__, __LINE__, cpdd);
-		fprintf(stderr, "フォルダ情報変更エラーです (%s) \n", cpinfo->dst);
+		print_error("utimensat", __FILE__, __LINE__, cpdd);
+		fprintf(stderr, "error : %s\n", cpinfo->dst);
 	}
 }
 
