@@ -26,6 +26,7 @@
 
 #include <errno.h>
 #include <fcntl.h>
+#include <limits.h>
 #include <linux/falloc.h>
 #include <pthread.h>
 #include <sys/mman.h>
@@ -35,27 +36,31 @@
 #define OPEN_CHECK_DST \
 {\
 	errno = 0;\
-	if((o_direct_write_flag == false) || (src_size_local < USE_O_DIRECT_SIZE))\
+	if((is_o_direct_write == false) || (src_size_local < USE_O_DIRECT_SIZE))\
 	{\
 		idst = open(cpinfo->dst, O_CREAT | O_WRONLY | O_TRUNC | O_NOFOLLOW, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);\
-		o_direct_open_dst = false;\
+		is_direct_io = false;\
 	}\
 	else \
 	{\
 		idst = open(cpinfo->dst, O_CREAT | O_WRONLY | O_TRUNC | O_NOFOLLOW | O_DIRECT, S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH);\
-		o_direct_open_dst = true;\
+		is_direct_io = true;\
 	}\
 \
 	if(idst == -1)\
 	{\
-		o_direct_open_dst = false;\
+		is_direct_io = false;\
 		print_error("open", __FILE__, __LINE__, cpdd);\
 	}\
 	else \
 	{\
 		write_now = cpinfo->dst;	/* signalハンドラ用 */\
 		open_dst = true;\
-		cpdd->mk_file_count++;\
+\
+		if(cpdd->mk_file_count <= ULLONG_MAX)\
+		{\
+			cpdd->mk_file_count++;\
+		}\
 	}\
 }
 
@@ -115,20 +120,15 @@ static void * do_md5_append(void *tmp);
 *******************************************************************************/
 void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 {
-	static _Bool filesystem_check = false;
-	static _Bool fallocate_support = false;
-	static _Bool o_direct_write_flag = false;
-	static _Bool verify_option_check = false;
+	static _Bool check_filesystem = false;
+	static _Bool support_fallocate = false;
+	static _Bool is_o_direct_write = false;
+	static _Bool check_verify_option = false;
 
-	static _Bool open_src;
-	static _Bool stat_src;
-	static _Bool open_dst;
-	static _Bool o_direct_open_dst;
-
-	stat_src = false;
-	open_src = false;
-	open_dst = false;
-	o_direct_open_dst = false;
+	_Bool open_src = false;
+	_Bool stat_src = false;
+	_Bool open_dst = false;
+	_Bool is_direct_io = false;
 
 	src_size_local = 0;
 	written_size[0] = 0;
@@ -159,17 +159,17 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 		psize = sysconf(_SC_PAGESIZE);
 	}
 
-	if(verify_option_check == false)
+	if(check_verify_option == false)
 	{
 		if(oparg->CHECK == V_FIVE)
 		{
 			verify_option = true;
 		}
 
-		verify_option_check = true;
+		check_verify_option = true;
 	}
 
-	if(filesystem_check == false)
+	if(check_filesystem == false)
 	{
 		char *tmp = cpinfo->dstfs;
 
@@ -184,16 +184,16 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 				((tmp[0] == 'o') && (tmp[1] == 'c') && (tmp[2] == 'f') && (tmp[2] == 's') && (tmp[2] == '2'))
 			)
 			{
-				fallocate_support = true;
+				support_fallocate = true;
 			}
 			else if((tmp[0] == 'x') && (tmp[1] == 'f') && (tmp[2] == 's'))
 			{
-				fallocate_support = true;
-				o_direct_write_flag = true;
+				support_fallocate = true;
+				is_o_direct_write = true;
 			}
 		}
 
-		filesystem_check = true;
+		check_filesystem = true;
 	}
 
 	if(cpinfo->copy_mode == SAME)
@@ -246,7 +246,11 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 		{
 			stat_src = true;
 			src_size_local = stat_buf.st_size;
-			cpdd->total_source += src_size_local;
+
+			if(src_size_local > 0)
+			{
+				cpdd->total_source += src_size_local;
+			}
 		}
 
 		if((cpinfo->file_test == false) || (oparg->WRITE_MODE == OVERWRITE))
@@ -307,7 +311,7 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 	 * ext4だと4秒弱で終わるフォルダのコピーが、
 	 * ntfsだと12秒はかかる（何もしないと6秒弱、pwriteだと8秒ほど、ftruncateは何もしない場合とほぼ同じ）。
 	 */
-	if((src_size_local > 0) && (open_dst == true) && (fallocate_support == true))
+	if((src_size_local > 0) && (open_dst == true) && (support_fallocate == true))
 	{
 		/* Ubuntu 11.04だとfallocateの第二引数は FALLOC_FL_KEEP_SIZE と FALLOC_FL_PUNCH_HOLE の二つ。linux/falloc.h のincludeが必要。 */
 		if(fallocate(idst, FALLOC_FL_KEEP_SIZE, 0, src_size_local) == -1)
@@ -349,7 +353,7 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 		/* 1048576 == 1024 * 1024 == 1MB */
 		if(1048576 > src_size_local)
 		{
-			if(o_direct_open_dst == false)
+			if(is_direct_io == false)
 			{
 				mw_small(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
 			}
@@ -360,7 +364,7 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 		}
 		else if(cpinfo->copy_mode == DIFFERENT)
 		{
-			if(o_direct_open_dst == false)
+			if(is_direct_io == false)
 			{
 				mw_thread(oparg, cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
 			}
@@ -371,7 +375,7 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 		}
 		else
 		{
-			if(o_direct_open_dst == false)
+			if(is_direct_io == false)
 			{
 				mw_big(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
 			}
@@ -381,7 +385,7 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 			}
 		}
 
-		if((o_direct_open_dst == true) && (src_size_local < total_write_local))
+		if((is_direct_io == true) && (src_size_local < total_write_local))
 		{
 			if(ftruncate(idst, src_size_local) == -1)
 			{
@@ -406,7 +410,7 @@ void read_write(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd)
 /*
  * ext4でfdatasyncするようにしたら、30倍遅くなった。
 
-	if((oparg->CHECK != NOT) && (fallocate_support == true))
+	if((oparg->CHECK != NOT) && (support_fallocate == true))
 	{
 		if(fdatasync(idst) == -1)
 		{
