@@ -91,7 +91,7 @@ int main(int argc, char **argv)
 	}
 
 	/* ファイルとリンクはコピー直後もしくはベリファイ直後に削除しているので、チェックするのはフォルダだけでよい。 */
-	if((oparg->M == MOVE) && (cpdd->dir_count > 0))
+	if((oparg->M == MOVE) && (oparg->CHECK_MODE == AFTER) && (cpdd->dir_count > 0))
 	{
 		delete_src(file_list, cpdd);
 	}
@@ -112,12 +112,12 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if(cpdd->link_count > 0)   { printf("Link   : %lld / %lld\n", cpdd->mk_link_count, cpdd->link_count); }
-	if(cpdd->dir_count > 0)    { printf("Dir    : %lld / %lld\n", cpdd->mk_dir_count, cpdd->dir_count); }
-	if(cpdd->total_source > 0) { printf("Source : %lld MB (%lld Bytes)\n", cpdd->total_source / (1024 * 1024), cpdd->total_source); }
-	if(cpdd->total_read > 0)   { printf("Read   : %lld MB (%lld Bytes)\n", cpdd->total_read / (1024 * 1024), cpdd->total_read); }
-	if(cpdd->total_write > 0)  { printf("Write  : %lld MB (%lld Bytes)\n", cpdd->total_write / (1024 * 1024), cpdd->total_write); }
-	if(cpdd->total_error > 0)  { printf("Error  : %lld\n", cpdd->total_error); }
+	if(cpdd->link_count > 0)   { printf("Link   : %llu / %llu\n", cpdd->mk_link_count, cpdd->link_count); }
+	if(cpdd->dir_count > 0)    { printf("Dir    : %llu / %llu\n", cpdd->mk_dir_count, cpdd->dir_count); }
+	if(cpdd->total_source > 0) { printf("Source : %llu MB (%llu Bytes)\n", cpdd->total_source / (1024 * 1024), cpdd->total_source); }
+	if(cpdd->total_read > 0)   { printf("Read   : %llu MB (%llu Bytes)\n", cpdd->total_read / (1024 * 1024), cpdd->total_read); }
+	if(cpdd->total_write > 0)  { printf("Write  : %llu MB (%llu Bytes)\n", cpdd->total_write / (1024 * 1024), cpdd->total_write); }
+	if(cpdd->total_error > 0)  { printf("Error  : %llu\n", cpdd->total_error); }
 
 	/*
 	 * close()する必要があるもの
